use std::borrow::BorrowMut;

use isahc::{ReadResponseExt, Request, RequestExt};
use serde::Deserialize;

use crate::{counter::count, BASE_URL};

#[derive(Deserialize, Debug)]
#[serde(untagged)]
enum PostsResponse {
    DanbooruPosts(Vec<DanbooruPost>),
    DanbooruError(DanbooruError),
}

#[derive(Deserialize, Debug)]
pub struct DanbooruPost {
    pub large_file_url: Option<String>,
    pub id: u64,
    pub file_size: Option<u64>,
}

#[derive(Deserialize, Debug)]
pub struct DanbooruError {
    pub error: String,
    pub message: String,
}

pub fn get_posts(tags: String, limit: u64) -> Result<Vec<DanbooruPost>, String> {
    let mut posts: Vec<DanbooruPost> = vec![];
    let mut count = count(tags.clone())?;

    if limit != 0 && count > limit {
        count = limit;
    };

    while (posts.len() as u64) < count {
        let id_part = if let Some(post) = posts.first() {
            format!("id:<={}", post.id)
        } else {
            "".into()
        };
        let mod_tags = format!("{} {}", tags, id_part);

        let posts_to_get = if count > 200 { 200 } else { count };

        let web_safe_tags: String =
            url::form_urlencoded::byte_serialize(mod_tags.as_bytes()).collect();
        let mut status_response = Request::get::<String>(format!(
            "{}/posts.json?tags={}&limit={}&page={}",
            BASE_URL,
            web_safe_tags,
            posts_to_get,
            (posts.len() / 200) as u64 + 1
        ))
        .body(())
        .expect("body error")
        .send();

        let text_try = match status_response.borrow_mut() {
            Ok(res) => res.text(),
            Err(err) => return Err(err.to_string()),
        };

        let text = match text_try {
            Ok(text) => text,
            Err(_) => return Err("Couldn't parse posts body".into()),
        };

        let parsed = match serde_json::from_str::<PostsResponse>(&text) {
            Ok(parsed) => parsed,
            Err(e) => return Err(format!("Couldn't parse posts text: {}", e.to_string()).into()),
        };

        let mut posts_response = match parsed {
            PostsResponse::DanbooruPosts(posts) => posts,
            PostsResponse::DanbooruError(err) => return Err(err.message),
        };

        if posts_response.len() == 0 {
            println!("Length of posts is 0, breaking");
            break;
        };

        posts.append(&mut posts_response);
        println!("Got {}/{} posts", posts.len(), count);
    }

    Ok(posts)
}
