use clap::Parser;
use std::fs::File;
use std::io::Write;
use std::path::Path;
use std::{fs, process};

pub const BASE_URL: &'static str = "https://danbooru.donmai.us";

mod counter;
mod downloader;
mod getter;
mod util;

use downloader::download;

use crate::util::filter_posts;

/// A bulk image downloader
#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
struct Args {
    /// List of tags separated by spaces
    tags: Vec<String>,

    /// Folder to output images into
    #[arg(short, long, default_value = "danbooru downloads")]
    out: String,

    /// Get first n posts from the query. Set to 0 for infinite
    #[arg(short, long, default_value_t = 0)]
    limit: u64,
}

fn main() {
    let mut args = Args::parse();

    if args.out.ends_with("/") {
        args.out.pop();
    };

    let posts = match getter::get_posts(args.tags.join(" "), args.limit) {
        Ok(posts) => posts,
        Err(e) => {
            println!("Error: {}", e);
            process::exit(1);
        }
    };

    let results_amount = posts.len();

    if results_amount == 0 {
        println!("The query returned no results")
    }

    // filter out posts we already have downloaded in the outfolder
    let (posts, downloaded, failed) = filter_posts(&args.out, posts);

    if posts.len() == 0 {
        println!("There is nothing to do.");
        let downloaded_text = format!(
            "You already have {} post{} downloaded",
            downloaded,
            if downloaded > 1 { "s" } else { "" }
        );
        let failed_text = format!(
            "{}{} post{} can't be accessed",
            if downloaded > 0 { " and " } else { "" },
            failed,
            if failed > 1 { "s" } else { "" }
        );
        println!(
            "{}{}",
            if downloaded > 0 {
                downloaded_text
            } else {
                "".into()
            },
            if failed > 0 { failed_text } else { "".into() }
        );
        process::exit(0);
    }

    let mut total_size: u64 = 0;
    for p in &posts {
        total_size += p.file_size;
    }

    println!(
        "You're about to download {:.2} MB ({} posts)",
        total_size / 1024 / 1024,
        posts.len(),
    );

    let confirmation = util::ask_confirm("Continue?");

    if !confirmation {
        println!("Exiting...");
        process::exit(0);
    };

    if !Path::new(&args.out).exists() {
        if fs::create_dir_all(&args.out).is_err() {
            println!("Couldn't create the folder");
            process::exit(1);
        };
    };

    let mut posts_got: u64 = 0;
    let mut downloaded: u64 = 0;

    for post in &posts {
        let download_bytes = download(post.file_url.as_str()).unwrap();

        let file_extension = match util::get_extension(post.file_url.as_str()) {
            Ok(extension) => extension,
            Err(_) => {
                println!("Failed to get file extension: {}", post.id);
                continue;
            }
        };

        let mut f = File::create(format!("{}/{}.{}", args.out, post.id, file_extension))
            .expect("create error");
        f.write_all(download_bytes.as_slice()).expect("write error");

        downloaded += post.file_size;
        posts_got += 1;
        println!(
            "Downloaded {}/{} posts ({}/{} MB)",
            posts_got,
            posts.len(),
            downloaded / 1024 / 1024,
            total_size / 1024 / 1024
        );
    }
}
